"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const Validator_1 = __importDefault(require("./Validator"));
const util_1 = require("../util");
const qs_1 = __importDefault(require("qs"));
const validator = Validator_1.default;
const UNPROCESSABLE_ENTITY = 422;
class BaseService {
    constructor(endpoint, parameters) {
        this.endpoint = endpoint;
        this.parameters = parameters;
        this.errors = Validator_1.default;
    }
    get $http() {
        return BaseService.$http;
    }
    get $errorProperty() {
        return BaseService.$errorProperty;
    }
    get $parsedQs() {
        return BaseService.$parsedQs;
    }
    all() {
        return this.submit('get');
    }
    find(id) {
        return this.submit('get', id);
    }
    post(payload, config) {
        return this.submit('post', '', payload, config);
    }
    store(payload, config) {
        return this.post(payload, config);
    }
    put(id, payload, config) {
        const parameter = id && !(0, util_1.isObject)(id) ? `/${id}` : '';
        const body = (0, util_1.isObject)(id) ? id : payload;
        const requestType = (0, util_1.hasFiles)(body) ? 'post' : 'put';
        if ((0, util_1.hasFiles)(body)) {
            Object.assign(body, { _method: 'put' });
        }
        return this.submit(requestType, parameter, body, config);
    }
    patch(id, payload, config) {
        const parameter = id && !(0, util_1.isObject)(id) ? `/${id}` : '';
        const body = (0, util_1.isObject)(id) ? id : payload;
        return this.submit('patch', parameter, body, config);
    }
    update(id, payload) {
        return this.patch(id, payload);
    }
    delete(id) {
        return this.submit('delete', `/${id}`);
    }
    remove(id) {
        return this.delete(id);
    }
    submit(method, parameter, form, config) {
        BaseService.__validateRequestType(method);
        this.beforeSubmit();
        return new Promise((resolve, reject) => {
            const data = (0, util_1.hasFiles)(form) ? (0, util_1.objectToFormData)(form) : form;
            const endpoint = parameter
                ? `/${this.endpoint}/${parameter}`
                : `/${this.endpoint}`;
            const url = this.__getParameterString((0, util_1.removeDoubleSlash)(endpoint));
            config = Object.assign({}, config, { url, data, method });
            this.$http(config)
                .then((response) => {
                this.onSuccess();
                resolve(response.data || {});
            })
                .catch((error) => {
                this.errors.processing = false;
                validator.processing = false;
                const { response } = error;
                if (response && response.status === UNPROCESSABLE_ENTITY) {
                    const { data } = response;
                    const errors = {};
                    Object.assign(errors, data[this.$errorProperty]);
                    this.onFail(errors);
                }
                reject(error);
            });
        });
    }
    __getParameterString(url) {
        const query = qs_1.default.stringify(this.parameters, {
            encode: false,
            skipNulls: true,
            addQueryPrefix: true,
        });
        return `${url}${query}`;
    }
    static __validateRequestType(requestType) {
        const requestTypes = [
            'get',
            'GET',
            'delete',
            'DELETE',
            'head',
            'HEAD',
            'options',
            'OPTIONS',
            'post',
            'POST',
            'put',
            'PUT',
            'patch',
            'PATCH',
        ];
        if (!requestTypes.includes(requestType)) {
            throw new Error(`\`${requestType}\` is not a valid request type, ` +
                `must be one of: \`${requestTypes.join('`, `')}\`.`);
        }
        return requestType;
    }
    setParameters(parameters) {
        Object.keys(parameters).forEach((key) => {
            this.parameters[key] = parameters[key];
        });
        return this;
    }
    setParameter(parameter, value) {
        if (!value) {
            const options = Object.assign({}, this.$parsedQs, {
                comma: true,
                allowDots: true,
                ignoreQueryPrefix: true,
            });
            const params = qs_1.default.parse(parameter, options);
            return this.setParameters(params);
        }
        this.parameters[parameter] = value;
        return this;
    }
    removeParameters(parameters = []) {
        if (!parameters || !parameters.length) {
            this.parameters = [];
        }
        else if ((0, util_1.isArray)(parameters)) {
            for (const parameter of parameters) {
                delete this.parameters[parameter];
            }
        }
        return this;
    }
    removeParameter(parameter) {
        delete this.parameters[parameter];
        return this;
    }
    onFail(errors) {
        this.errors.fill(errors);
        validator.fill(errors);
    }
    beforeSubmit() {
        if (!this.$http) {
            throw new Error('Vue Axios Http, No http library provided.');
        }
        this.errors.flush();
        this.errors.processing = true;
        this.errors.successful = false;
        validator.flush();
        validator.processing = true;
        validator.successful = false;
    }
    onSuccess() {
        this.errors.processing = false;
        this.errors.successful = true;
        validator.processing = false;
        validator.successful = true;
    }
}
BaseService.$errorProperty = 'errors';
BaseService.$parsedQs = {
    comma: true,
    allowDots: true,
    ignoreQueryPrefix: true,
};
exports.default = BaseService;
